#! /usr/bin/python3

################################################################################
# flexiWAN SD-WAN software - flexiEdge, flexiManage.
# For more information go to https://flexiwan.com
#
# Copyright (C) 2021  flexiWAN Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
################################################################################

import PAM
import cherrypy


def check_login(username, password):
    """Checks Linux login credentials using PAM

    Args:
        username (string): Linux username
        password (string): Linux password (plaintext)
    Returns:
        0 if success, otherwise failure
    """
    print(f"Login: check_login {username}")

    def pam_conv(auth, query_list, userData):
        resp = []
        resp.append((password, 0))
        return resp

    res = PAM.PAM_AUTH_ERR
    service = "passwd"

    auth = PAM.pam()
    auth.start(service)
    auth.set_item(PAM.PAM_USER, username)
    auth.set_item(PAM.PAM_CONV, pam_conv)
    try:
        res = auth.authenticate()
        res = auth.acct_mgmt()
        print("acct_mgmt %s" % (res))
    except PAM.error as resp:
        desc = resp.args[0]
        err = resp.args[1]
        print("Authentication failure (%s) (%s)" % (desc, err))
        res = err
    except:
        print("Authentication failure, internal error")
        res = -2
    else:
        print("Authentication success")
        res = 0

    del auth
    return res
