#! /usr/bin/python3

################################################################################
# flexiWAN SD-WAN software - flexiEdge, flexiManage.
# For more information go to https://flexiwan.com
#
# Copyright (C) 2021  flexiWAN Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
################################################################################

import os, os.path

import cherrypy
import json
import sys
import access


def is_python2():
    """Checks if it is Python 2 version.

    :returns: 'True' if Python2 and 'False' otherwise.
    """
    ret = True if sys.version_info < (3, 0) else False
    return ret


@cherrypy.tools.json_out()
def check_access(token_file=""):
    # skip OPTIONS
    if cherrypy.request.method in ("OPTIONS"):
        return

    auth_header = cherrypy.request.headers.get("Authorization", "")
    if auth_header == "":
        print("Missing authorization header")
        cherrypy.request.handler = None
        cherrypy.response.status = 401
        return
    auth_header = auth_header.split()
    if not len(auth_header) == 2:
        print("Invalid authorization header format")
        raise cherrypy.HTTPError(401, message="Invalid auth token")
    if not auth_header[0] == "Bearer":
        print("Invalid authorization header prefix")
        raise cherrypy.HTTPError(401, message="Invalid auth token")

    if token_file == "":
        token = auth_header[1]
        if not access.validate_access_token(token):
            cherrypy.request.handler = None
            cherrypy.response.status = 401
        return

    print("Printing token file %s" % (token_file))
    if not os.path.isfile(token_file):
        raise cherrypy.HTTPError(401, message="Invalid auth token")
    # read token file
    token = ""
    try:
        with open(token_file, "r") as f:
            token = f.readline().rstrip()
    except:
        err = "Failed to load token from %s: %s (%s)" % (
            token_file,
            format(sys.exc_info()[1]),
            format(sys.exc_info()[0]),
        )
        raise cherrypy.HTTPError(401, message=err)
    if not auth_header[1] == token:
        print("Invalid authorization token")
        raise cherrypy.HTTPError(401, message="Invalid auth token")


def check_options():
    """
    Checks that request method is OPTIONS and responds accordingly
    """
    if cherrypy.request.method in ("OPTIONS"):
        cherrypy.request.handler = None
        cherrypy.response.status = 204


def check_options_after():
    """
    Checks that request method is OPTIONS and responds accordingly
    """
    # Get the request origin and return it in the Access-Control-Allow-Origin response header
    cherrypy.response.headers[
        "Access-Control-Allow-Origin"
    ] = cherrypy.request.headers.get("Origin", "*")

    if cherrypy.request.method in ("OPTIONS"):
        print(
            "[check_options_after]Current response status %s" % cherrypy.response.status
        )
        print("[check_options_after] Current response body %s" % cherrypy.response.body)
        cherrypy.response.status = 204


def get_error_response(message):
    error = {"error": message}
    if is_python2():
        return bytes(json.dumps(error))
    return bytes(json.dumps(error), "utf-8")


cherrypy.tools.check_access = cherrypy.Tool("before_request_body", check_access)
cherrypy.tools.check_options = cherrypy.Tool("on_start_resource", check_options)
cherrypy.tools.check_options_after = cherrypy.Tool(
    "before_finalize", check_options_after
)
