#! /usr/bin/python3

################################################################################
# flexiWAN SD-WAN software - flexiEdge, flexiManage.
# For more information go to https://flexiwan.com
#
# Copyright (C) 2021  flexiWAN Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
################################################################################

import cherrypy
from mediator import Mediator


@cherrypy.expose
class InterfacesModule:
    def __init__(self):
        self.mediator = Mediator()

    def OPTIONS(self):
        cherrypy.response.status = 204
        return ""

    @cherrypy.tools.accept(media="application/json")
    @cherrypy.tools.json_out()
    def GET(self):
        status, response = self.mediator.get_interfaces()
        if status == True:
            return response
        cherrypy.response.status = 500
        return {"error": response}

    @cherrypy.tools.accept(media="application/json")
    @cherrypy.tools.json_in()
    @cherrypy.tools.json_out()
    def PUT(self):
        data = cherrypy.request.json
        print(data)

        if not isinstance(data, list):
            status, update_response = self.mediator.update_device_interfaces(data)
            if status is True:
                return update_response
            cherrypy.response.status = 500
            return {"error": update_response}

        update_responses = []
        for interface in data:
            status, update_response = self.mediator.update_device_interfaces(interface)
            if status is False:
                cherrypy.response.status = 500
                return {"error": update_response}
            update_responses.append(update_response)
        return update_responses
