#! /usr/bin/python3

################################################################################
# flexiWAN SD-WAN software - flexiEdge, flexiManage.
# For more information go to https://flexiwan.com
#
# Copyright (C) 2021  flexiWAN Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
################################################################################

import cherrypy
from mediator import Mediator
import auth_helper
import base64
import access


@cherrypy.expose
class LoginModule:
    def __init__(self):
        self.mediator = Mediator()

    @cherrypy.tools.accept(media="text/plain")
    @cherrypy.tools.json_in()
    @cherrypy.tools.json_out()
    def POST(self):
        creds = cherrypy.request.json
        username = creds.get("username")
        password = creds.get("password")
        try:
            password = _decode_password(password)
        except:
            print("_decode_password call failed")
            cherrypy.response.status = 401
            return {"error": "Password or username is incorrect"}
        res = auth_helper.check_login(username, password)
        if res == 0:
            # Authenticated
            # Delete expired tokens
            access.check_expired_access_tokens()
            # Generate and assign new access token
            access_token = access.assign_access_token(username)
            return {
                "name": username,
                "status": "logged in",
                "access_token": access_token,
                "token_type": "Bearer",
            }
        if res == 12:
            cherrypy.response.status = 401
            return {"error": "Password expired"}
        cherrypy.response.status = 401
        return {"error": "Password or username is incorrect"}

    @cherrypy.tools.accept(media="text/plain")
    @cherrypy.tools.json_in()
    @cherrypy.tools.json_out()
    def PUT(self):
        creds = cherrypy.request.json
        username = creds.get("username")
        password = creds.get("password")
        new_password = creds.get("newPassword")
        try:
            print("Login: decoding password")
            password = _decode_password(password)
            new_password = _decode_password(new_password)
        except:
            print("Login: error while decoding password")
            cherrypy.response.status = 500
            return {"error": "Failed to modify password", "username": username}
        # check current credentials
        print("Login: checking login")
        res = auth_helper.check_login(username, password)
        if res != 0 and res != 12:
            print("Login: error while checking login")
            cherrypy.response.status = 500
            return {"error": "Failed to modify password", "username": username}
        # attempt to change the password
        status, response = self.mediator.change_password(
            creds.get("username"), new_password
        )
        if status == True:
            return {
                "name": creds.get("username"),
                "status": "Password updated successfully",
            }
        cherrypy.response.status = 500
        return {"error": response}

    def OPTIONS(self, id, method):
        cherrypy.response.status = 204
        return ""


def _decode_password(password):
    base_64 = base64.b64decode(password)
    return base_64.decode("utf-8")
