#! /usr/bin/python3

################################################################################
# flexiWAN SD-WAN software - flexiEdge, flexiManage.
# For more information go to https://flexiwan.com
#
# Copyright (C) 2021  flexiWAN Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
################################################################################

import cherrypy
from mediator import Mediator


@cherrypy.expose
class SystemCheckersModule:
    def __init__(self):
        self.mediator = Mediator()

    @cherrypy.tools.accept(media="text/plain")
    @cherrypy.tools.json_out()
    def GET(self):
        status, response = self.mediator.get_system_checkers()
        if status == True:
            return response
        cherrypy.response.status = 500
        return {"error": response}

    @cherrypy.popargs("checker_name")
    def OPTIONS(self, checker_name=None):
        cherrypy.response.status = 204
        return ""

    @cherrypy.popargs("checker_name")
    @cherrypy.tools.accept(media="application/json")
    @cherrypy.tools.json_in()
    @cherrypy.tools.json_out()
    def PUT(self, checker_name=None):
        data = cherrypy.request.json
        print(data)
        auto_fix = data.get("autoFix")
        if checker_name is None:
            print("Running all system checkers")
            status, response = self.mediator.run_system_checkers(auto_fix)
            if status == True:
                return response
            cherrypy.response.status = 500
            return {"error": response}

        print("Running single system checker %s" % (checker_name))
        status, response = self.mediator.run_system_checker(checker_name, auto_fix)
        if status == True:
            return response
        cherrypy.response.status = 500
        return {"error": response}
