#!/usr/bin/env python
import subprocess
import pwd
import os
import sys

# https://stackoverflow.com/questions/1770209/run-child-processes-as-different-user-from-a-long-running-python-process
# https://stackoverflow.com/questions/567542/running-a-command-as-a-super-user-from-a-python-script


def run_command_as(user_name, sudo_password, command):
    print("user_name", user_name)
    print("command", command)
    command = command.split()

    # copy user environment
    pw_record = pwd.getpwnam(user_name)
    user_name = pw_record.pw_name
    user_home_dir = pw_record.pw_dir
    user_uid = pw_record.pw_uid
    user_gid = pw_record.pw_gid
    env = os.environ.copy()
    env["HOME"] = user_home_dir
    env["LOGNAME"] = user_name
    env["USER"] = user_name

    # First command echoes sudo password into the stdout
    # in order to be piped into the second command with actual
    # command to be run as sudo.
    cmd1 = subprocess.Popen(
        ["echo", sudo_password],
        stdout=subprocess.PIPE,
        preexec_fn=demote(user_uid, user_gid, user_name),
        env=env,
    )
    cmd2 = subprocess.Popen(
        ["sudo", "-S"] + command,
        stdin=cmd1.stdout,
        stdout=subprocess.PIPE,
        preexec_fn=demote(user_uid, user_gid, user_name),
        env=env,
    )

    result = cmd2.wait()
    print("Command completed", command)
    print("result", result)
    return result


def demote(user_uid, user_gid, user_name):
    def result():
        print("starting demotion")
        # os.setgid(user_gid)
        # as setgid uses only primary gid, so in order to use all of the user groups
        # (including sudo if available), then initgroups is to the rescue. Otherwise,
        # most likely the "username is not in the sudoers file" error is returned.
        os.initgroups(user_name, user_gid)
        os.setuid(user_uid)
        print("finished demotion")

    return result


def main(my_args=None):
    if my_args is None:
        my_args = sys.argv[1:]
        # user_name = "test_with_sudo"
        # sudo_password = "t"
        # command = "fwagent reset -s"
    user_name, sudo_password, command = my_args[:3]
    return run_command_as(user_name, sudo_password, command)


if __name__ == "__main__":
    main()
