const request = require("../commonTests");

describe("EdgeUI API endpoints tests with no or token based access", () => {
  beforeEach(function () {
    // console.log('Input user details!');
    user1Details = {
      username: request.edgeUi.username,
      password: request.edgeUi.password,
    };

    user2Details = {
      username: request.edgeUi.username2,
      password: request.edgeUi.password2,
    };
  });

  login = async (userDetails) => {
    const res = await request.request
      .post(`login`)
      .send(userDetails)
      .expect(200);

    expect(res.body).toBeDefined();
    const body = res.body;
    expect(body).toHaveProperty("name", userDetails.username);
    expect(body).toHaveProperty("status", "logged in");
    expect(body).toHaveProperty("access_token");
    expect(body).toHaveProperty("token_type", "Bearer");

    const access_token = res.body.access_token;
    console.log(access_token);
    return access_token;
  };

  it("Should Login Success without bearer token", async () => {
    await login(user1Details);
  });

  it("Should generate new bearer token with every login", async () => {
    const first_token = await login(user1Details);
    const second_token = await login(user1Details);

    expect(second_token).not.toEqual(first_token);
  });

  it("Should respond with success to Get Token API request with authorization", async () => {
    const access_token = await login(user1Details);

    const res2 = await request.request
      .get(`token`)
      .set("Authorization", `Bearer ${access_token}`)
      .expect(200);

    expect(res2.body).toBeDefined();
    const body2 = res2.body;
    expect(body2).toHaveProperty("deviceToken");
  });

  it("Should respond with success to Get fwdump API request with authorization", async () => {
    const access_token = await login(user1Details);

    const res = await request.request
      .get(`dump`)
      .set("Authorization", `Bearer ${access_token}`)
      .expect(200);

    expect(res.header).toHaveProperty("content-disposition");
    expect(res.body).toBeDefined();
  });

  it("Should create two bearer tokens from two logins", async () => {
    // The test assumes at least two users exist on target machine
    const access_token1 = await login(user1Details);

    const res2 = await request.request
      .get(`token`)
      .set("Authorization", `Bearer ${access_token1}`)
      .expect(200);

    expect(res2.body).toBeDefined();
    const body2 = res2.body;
    expect(body2).toHaveProperty("deviceToken");

    const access_token2 = await login(user2Details);

    const res4 = await request.request
      .get(`token`)
      .set("Authorization", `Bearer ${access_token2}`)
      .expect(200);

    expect(res4.body).toBeDefined();
    const body4 = res4.body;
    expect(body4).toHaveProperty("deviceToken");
  });

  it.skip(
    "Should respond with error to Get Token API request with authorization when token expired",
    async () => {
      const res = await request.request
        .post(`login`)
        .send(user1Details)
        .expect(200);

      expect(res.body).toBeDefined();
      const body = res.body;
      const access_token = body.access_token;

      await new Promise((r) => setTimeout(r, 61 * 1000));

      await request.request
        .get(`token`)
        .set("Authorization", `Bearer ${access_token}`)
        .expect(401);
    },
    61 * 1000 + 5000
  );

  it("Should respond with success to Get Device API request", async () => {
    const access_token = await login(user1Details);

    const res = await request.request
      .get(`device`)
      .set("Authorization", `Bearer ${access_token}`)
      .expect(200);

    expect(res.body).toBeDefined();
    const { body } = res;
    expect(body).toHaveProperty("deviceManagement");
    expect(body).toHaveProperty("deviceSerialNumber");
    expect(body).toHaveProperty("hostname");
    expect(body).toHaveProperty("machineId");
    expect(body).toHaveProperty("vRouter");
    expect(body).toHaveProperty("versions");
  });

  it.each`
    endpoint
    ${"device"}
    ${"token"}
    ${"interfaces"}
    ${"systemcheckers"}
    ${"logs"}
    ${"connectivity"}
  `(
    "Should respond with error to '$endpoint' API request with no authorization",
    async ({ endpoint }) => {
      await request.request.get(endpoint).expect(401);
    }
  );

  it.each`
    endpoint
    ${"device"}
    ${"token"}
    ${"interfaces"}
    ${"systemcheckers"}
    ${"logs"}
    ${"connectivity"}
  `(
    "Should respond with error to '$endpoint' API request with invalid authorization",
    async ({ endpoint }) => {
      await request.request
        .get(`token`)
        .set("Authorization", `Bearer unknown`)
        .expect(401);
    }
  );
});
