import json
import os
import sys
import pytest

backend_path = os.path.realpath(
    os.path.join(os.path.dirname(os.path.realpath(__file__)), "../")
)
print("Backend path %s" % backend_path)
sys.path.append(backend_path)

from mediator import Mediator


class TestToken:
    @pytest.mark.parametrize(
        "input",
        [
            "",
            json.dumps({"unknown": "unknown"}),
        ],
    )
    def test_update_device_tokens_failure_invalid_input(self, input):
        expected = "Invalid input"
        mediator = Mediator()

        status, response = mediator.update_device_tokens(input)
        assert status == False
        assert response == expected

    @pytest.mark.parametrize(
        "token_file,expected",
        [
            (None, {"deviceToken": ""}),
            (
                os.path.join(
                    os.path.dirname(os.path.realpath(__file__)),
                    "input/not_existing_token.txt",
                ),
                {"deviceToken": ""},
            ),
            (
                os.path.join(
                    os.path.dirname(os.path.realpath(__file__)), "input/token.txt"
                ),
                {"deviceToken": "The answer to life, the universe, and everything"},
            ),
        ],
    )
    def test_get_device_tokens_success_different_inputs(self, token_file, expected):
        mediator = Mediator()

        def get_token_file():
            return token_file

        status, response = mediator.get_device_token(get_token_file)
        assert status == True
        assert response == expected
