#! /usr/bin/python3

################################################################################
# flexiWAN SD-WAN software - flexiEdge, flexiManage.
# For more information go to https://flexiwan.com
#
# Copyright (C) 2021  flexiWAN Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
################################################################################


import subprocess
import os
import yaml
import cherrypy
from cherrypy._cpdispatch import Dispatcher

from token_module import TokenModule
from logs_module import LogsModule
from dump_module import DumpModule
from uuid_module import UuidModule
from device_module import DeviceModule
from login_module import LoginModule
from interfaces_module import InterfacesModule
from system_checkers_module import SystemCheckersModule
from connectivity_module import ConnectivityModule
from jobs_module import JobsModule

import custom_tools


def check_edgeui_enabled():
    """checks whether edge ui is enable in configuration file. If not enabled, stops daemon and exits"""
    # checking whether edge ui is enabled in configuration
    filename = "/etc/flexiwan/agent/fwagent_conf.yaml"
    with open(filename, "r") as conf_file:
        conf = yaml.load(conf_file, Loader=yaml.SafeLoader)
    edgeui_conf = conf.get("edgeui", {})
    if not edgeui_conf:
        return
    res = edgeui_conf.get("enabled", True)
    print("check_edgeui_enabled", res)
    if not res:
        print("edgeUI is disabled, exiting")
        subprocess.check_output("systemctl stop flexiwan-router-ui", shell=True)
        os._exit(1)


class OptionsDispatcher(Dispatcher):
    @cherrypy.tools.json_out()
    def __call__(self, path_info):
        print("Inside dispatcher")
        method = cherrypy.request.method
        print("Checking options for method %s" % (method))
        if cherrypy.request.method in ("OPTIONS"):
            cherrypy.response.status = 204
            return ""
        return Dispatcher.__call__(self, path_info.lower())


class WebApp(object):
    @cherrypy.expose
    def index(self):
        return open("./client/index.html")

    @cherrypy.expose
    @cherrypy.tools.json_out()
    def default(self, tag):
        cherrypy.response.status = 404
        return {"error": "Requested URL not found"}


class Www(object):
    @cherrypy.expose
    def index(self):
        raise cherrypy.InternalRedirect("/")

    @cherrypy.expose
    def default(self, *args, **kwargs):
        return self.index()


if __name__ == "__main__":

    check_edgeui_enabled()

    webapp = WebApp()
    webapp.device = DeviceModule()
    webapp.token = TokenModule()
    webapp.logs = LogsModule()
    webapp.dump = DumpModule()
    webapp.uuid = UuidModule()
    webapp.interfaces = InterfacesModule()
    webapp.login = LoginModule()
    webapp.systemcheckers = SystemCheckersModule()
    webapp.connectivity = ConnectivityModule()
    webapp.jobs = JobsModule()

    cherrypy.tree.mount(Www(), "/", "edgeui.conf")
    cherrypy.quickstart(webapp, "/api", "server.conf")
